package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.blackvein.quests.quests.IQuest;
import me.blackvein.quests.quests.IStage;
import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.NPC;
import net.citizensnpcs.api.npc.NPCRegistry;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import org.bukkit.inventory.ItemStack;

import java.util.List;
import java.util.UUID;

/**
 * A class to generate a string containing all information about a quest's stages
 */
public class QuestStagesInfoGenerator {

    private final IQuest quest;

    /**
     * Instantiates a new quest stages info generator
     *
     * @param quest <p>The quest to generate information about</p>
     */
    public QuestStagesInfoGenerator(IQuest quest) {
        this.quest = quest;
    }

    /**
     * Gets information about a quest's stages
     *
     * @return <p>A string with information about the quest's stages</p>
     */
    public String getQuestStagesInfo() {
        StringBuilder questInfo = new StringBuilder();
        NPCRegistry registry = CitizensAPI.getNPCRegistry();
        int stageCounter = 1;
        questInfo.append("<b>Stages:</b><ul>");
        for (IStage stage : quest.getStages()) {
            questInfo.append("<li><b>Stage ").append(stageCounter).append(" tasks:</b><ul>");
            int mobTypes = stage.getMobsToKill().size();
            for (int i = 0; i < mobTypes; i++) {
                questInfo.append("<li>Kill ").append(QuestsHelper.normalizeName(stage.getMobsToKill().get(i).name())).append(
                        " X ").append(stage.getMobNumToKill().get(i)).append("</li>");
            }
            int deliveries = stage.getItemDeliveryTargets().size();
            for (int i = 0; i < deliveries; i++) {
                NPC npc = registry.getByUniqueId(stage.getItemDeliveryTargets().get(i));
                questInfo.append("<li>Deliver ").append(QuestsHelper.getItemStackString(stage.getItemsToDeliver().get(i))).append(
                        " to ").append(npc.getName()).append("</li>");
            }
            if (stage.getFishToCatch() != null) {
                questInfo.append("<li>Catch ").append(stage.getFishToCatch()).append(" fish").append("</li>");
            }
            for (UUID npcId : stage.getNpcsToKill()) {
                questInfo.append("<li>Kill NPC ").append(registry.getByUniqueId(npcId).getName()).append("</li>");
            }

            questInfo.append(getQuestItemsTaskString(stage.getBlocksToBreak(), "<li>Break ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getBlocksToCut(), "<li>Cut ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getBlocksToDamage(), "<li>Damage ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getBlocksToUse(), "<li>Use ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getBlocksToPlace(), "<li>Place ")).append("</li>");

            questInfo.append(getQuestItemsTaskString(stage.getItemsToBrew(), "<li>Brew ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getItemsToConsume(), "<li>Consume ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getItemsToCraft(), "<li>Craft ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getItemsToEnchant(), "<li>Enchant ")).append("</li>");
            questInfo.append(getQuestItemsTaskString(stage.getItemsToSmelt(), "<li>Smelt ")).append("</li>");

            int sheepTypes = stage.getSheepToShear().size();
            for (int i = 0; i < sheepTypes; i++) {
                questInfo.append("<li>Shear ").append(stage.getSheepNumToShear().get(i)).append(" ").append(
                        QuestsHelper.normalizeName(stage.getSheepToShear().get(i).name())).append(" sheep").append("</li>");
            }
            if (stage.getCowsToMilk() != null) {
                questInfo.append("<li>Milk ").append(stage.getCowsToMilk()).append(" cows").append("</li>");
            }

            int mobTamingEntries = stage.getMobsToTame().size();
            for (int i = 0; i < mobTamingEntries; i++) {
                questInfo.append("<li>Tame ").append(stage.getMobNumToTame().get(i)).append(" ").append(
                        QuestsHelper.normalizeName(stage.getMobsToTame().get(i).name())).append("</li>");
            }

            questInfo.append("</ul></li>");
            stageCounter++;
        }
        questInfo.append("</ul>");
        return questInfo.toString();
    }

    /**
     * Gets a string to display a quest task involving some action on an item
     *
     * @param items       <p>The items that are part of the task</p>
     * @param explanation <p>The explanation of what the player needs to do with the items</p>
     * @return <p>A string describing the necessary tasks</p>
     */
    private String getQuestItemsTaskString(List<ItemStack> items, String explanation) {
        StringBuilder questInfo = new StringBuilder();
        for (ItemStack itemStack : items) {
            questInfo.append(explanation).append(QuestsHelper.getItemStackString(itemStack));
        }
        return questInfo.toString();
    }

}
