package net.knarcraft.dynmapcitizens.util;

import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.handler.VaultHandler;
import net.knarcraft.dynmapcitizens.handler.trait.quests.QuestNPCType;
import org.bukkit.inventory.ItemStack;

/**
 * A helper class for various quests-related tasks
 */
public class QuestsHelper {

    /**
     * Gets an item stack string with the first character upper-cased
     *
     * @param itemStack <p>The item stack to get the string for</p>
     * @return <p>A string describing the item stack</p>
     */
    public static String getUpperCasedItemStackString(ItemStack itemStack) {
        return uppercaseFirst(getItemStackString(itemStack));
    }

    /**
     * Gets the proper string representation of an item stack
     *
     * @param itemStack <p>The item stack to print</p>
     * @return <p>The string representation of the item stack</p>
     */
    public static String getItemStackString(ItemStack itemStack) {
        return normalizeName(itemStack.getType().name()) + " x " + itemStack.getAmount();
    }

    /**
     * Makes the first character in a string uppercase
     *
     * @param string <p>The string to run on</p>
     * @return <p>The same string, with the first character converted to uppercase</p>
     */
    private static String uppercaseFirst(String string) {
        return string.substring(0, 1).toUpperCase() + string.substring(1);
    }

    /**
     * Normalizes an internal name to make it human-readable
     *
     * @param name <p>The name to normalize</p>
     * @return <p>The normalized name</p>
     */
    public static String normalizeName(String name) {
        return name.toLowerCase().replace("_", " ");
    }

    /**
     * Gets the currency to print for the given amount of money
     *
     * @param money <p>The amount to pay/use</p>
     * @return <p>The currency name to use</p>
     */
    public static String getCurrency(double money) {
        VaultHandler vaultHandler = DynmapCitizens.getInstance().getVaultHandler();
        if (vaultHandler.isEnabled()) {
            return vaultHandler.getCurrency(money != 1);
        } else {
            return "money";
        }
    }

    /**
     * Gets the marker title to use for the given quest NPC type
     *
     * @param type <p>The type of marker to get the title for</p>
     * @return <p>The title to use for the marker</p>
     */
    public static String getMarkerTitle(QuestNPCType type) {
        return switch (type) {
            case GIVER -> "Quest Start NPC: ";
            case INTERACT -> "Quest Interact NPC: ";
            case DELIVER -> "Quest Deliver NPC: ";
            case KILL -> "Quest Kill NPC: ";
            case CHAIN -> "Quest Chain NPC: ";
        };
    }

}
